unit DoTests;

interface

	uses
		Permute;

	procedure DoTestSimple;

	procedure DoBackward (theYVect: VectorPtr;
									isStepwise: Boolean);

	procedure DoForward (isStepwise: Boolean);

	procedure DoStepwise;

implementation

	procedure DoTestSimple;
		var
			i: Integer;
			tempStr: Str255;
			correct: Boolean;
			lesDescr: ChoiceArray;
	begin
		for i := 1 to nbDescr do
			begin
				NumToString(i, tempStr);
				lesDescr[i] := Concat('Variable ', tempStr);
			end;
		correct := GetIndependentChoice('Select the variables to be included in model:', nbDescr, lesDescr, varSelectionnees);

		DoPermTest(Y, X, B, addVect, varSelectionnees, nbPerm, 0, NO_ALPHA, PRINT, NO_PARTIAL, '');

	end;{ procedure DoTestSimple }

	procedure DoBackward (theYVect: VectorPtr;
									isStepwise: Boolean);
		function AllIncludedSig: Boolean;
			var
				j: Integer;
				dumBool: Boolean;
				petitNbDescr: Integer;
				tempStr: Str255;
		begin
			dumBool := True;
			petitNbDescr := 0;
			for j := 1 to nbDescr do
				if varSelectionnees[j] then
					petitNbDescr := petitNbDescr + 1;
			for j := 1 to petitNbDescr do
				if (addVect.probBonf[j] > pToRemove) then
					dumBool := False;{au moins une valeur n'est pas significative}
			AllIncludedSig := dumBool;
		end;{function AllIncludedSig}

		var
			i, j, etape, plusGrandIndex, nbDescrChoisis: Integer;
			plusGrandeProb: Real;
			tempStr: Str255;
			correct: Boolean;
			lesDescr: ChoiceArray;
			tempY: VectorPtr;
	begin
		tempY := VectorPtr(NewPtr(SizeOf(Vector)));

{ En cas de DTP ou TPT, quand on modifie la matrice Y, il faut re-remplir cette derniere }
{ avec les donnees originales.  Ceci est accompli en les conservant dans 'tempY'.}
		for i := 1 to nbValeurs do
			tempY^[i] := theYVect^[i];

		WriteLn(outputFile, '-----------------------------');
		WriteLn(outputFile, 'Backward Selection procedure:');
		WriteLn(outputFile, '-----------------------------');
		WriteLn(outputFile);

		if isStepwise then
			begin
				nbDescrChoisis := 0;
				for i := 1 to nbDescr do
					if varSelectionnees[i] then
						nbDescrChoisis := nbDescrChoisis + 1;
			end{if}
		else
			begin
				pToRemove := SelectSignificanceLevel('Select the p-to-remove value', 2);

				for i := 1 to nbDescr do
					begin
						NumToString(i, tempStr);
						lesDescr[i] := Concat('Variable ', tempStr);
					end;
				tempStr := 'Select the variables to be included in the procedure:';
				correct := GetIndependentChoice(tempStr, nbDescr, lesDescr, varSelectionnees);
				nbDescrChoisis := 0;
				for i := 1 to nbDescr do
					if varSelectionnees[i] then
						nbDescrChoisis := nbDescrChoisis + 1;
				if (nbDescrChoisis < nbDescr) then
					begin
						Write(outputFile, 'CAUTION! ---> Backward selection procedure with only ');
						WriteLn(outputFile, nbDescrChoisis : 1, ' variable(s) out of ', nbDescr : 1);
						WriteLn(outputFile);
					end;
				WriteLn(outputFile, 'p to remove: ', pToRemove : 5 : 3);
				WriteLn(outputFile);
			end;{else}

		etape := 1;

		if isStepwise then
			tempStr := 'Testing backward significance'
		else
			tempStr := '';
		DoPermTest(theYVect, X, B, addVect, varSelectionnees, nbPerm, etape, pToRemove, PRINT, NO_PARTIAL, tempStr);
		TestQuit('Backward Selection');

{ Recuperer les donnees originales du vecteur Y, qui sont dans tempY: }
		for i := 1 to nbValeurs do
			theYVect^[i] := tempY^[i];

{Examiner le vecteur B pour voir s'il est significatif}
{ En fait, les probabilites sont celles du t associe au b; voir DoPermTest }
		while (not AllIncludedSig) and (nbDescrChoisis > 1) do
			begin
				j := 0;
				plusGrandeProb := -1;{On n'a jamais vu une prob < 0, tant mieux!}
				for i := 1 to nbDescr do
					if varSelectionnees[i] then
						begin
							j := j + 1;
							if addVect.probBonf[j] > plusGrandeProb then
								begin
									plusGrandeProb := addVect.probBonf[j];
									plusGrandIndex := i;
								end;{if addVect.probBonf[j]}
						end;{if varSelectionnees[i]}
{ Il y a dans plusGrandIndex l'index de la variable la moins significative }
				varSelectionnees[plusGrandIndex] := False;
				nbDescrChoisis := nbDescrChoisis - 1;
				etape := etape + 1;
				if isStepwise then
					begin
						NumToString(etape, tempStr);
						tempStr := Concat('Testing backward significance, step ', tempStr);
					end
				else
					tempStr := '';

				DoPermTest(theYVect, X, B, addVect, varSelectionnees, nbPerm, etape, pToRemove, PRINT, NO_PARTIAL, tempStr);
				TestQuit('Backward Selection');

{ Recuperer les donnees originales du vecteur Y, qui sont dans tempY: }
				for i := 1 to nbValeurs do
					theYVect^[i] := tempY^[i];

			end;{while not AllIncludedSig}
		WriteLn(outputFile, '---------------------------------');
		WriteLn(outputFile, 'End Backward Selection procedure.');
		WriteLn(outputFile, '---------------------------------');
		WriteLn(outputFile);

		Dispose(tempY);
	end;{ procedure DoBackward }

	procedure DoForward (isStepwise: Boolean);
		var
			tempY, tempZ: VectorPtr;
			tempX: MatrixPtr;
			unB, bVarSel: SmallVectorPtr;
			unAddVect, addVectRMaxSig: MemPermuteVect;
			i, j, k, etape, indiceRMaxSig, nbDecimales, currVarIndex: Integer;
			varRemoved, varBeforeAndAfter, varSelTemp: ICReply;
			probMinB, probCurrB, probMinRSq, probCurrRSq, rSqMax: Double;
			conditionArret: Boolean;
			str1, str2: Str255;
	begin
		tempX := MatrixPtr(NewPtr(SizeOf(Matrix)));
		tempY := VectorPtr(NewPtr(SizeOf(Vector)));
		tempZ := VectorPtr(NewPtr(SizeOf(Vector)));
		unB := SmallVectorPtr(NewPtr(SizeOf(SmallVector)));
		bVarSel := SmallVectorPtr(NewPtr(SizeOf(SmallVector)));

		if not isStepwise then
			pToEnter := SelectSignificanceLevel('Select the p-to-enter value', 2);

		WriteLn(outputFile, '----------------------------');
		WriteLn(outputFile, 'Forward Selection procedure:');
		WriteLn(outputFile, '----------------------------');
		WriteLn(outputFile);
		if not isStepwise then
			begin
				WriteLn(outputFile, 'p to enter: ', pToEnter : 5 : 3);
				WriteLn(outputFile);
			end;{if}

		for i := 1 to nbValeurs do
			begin
				tempY^[i] := Y^[i];
				for j := 1 to nbDescr do
					tempX^[i, j] := X^[i, j];
			end;{for i}

		etape := 0;
{ Aucune variable selectionnee, ni enlevee (stepwise) dans un premier temps }
		for i := 1 to MAX_DESC do
			begin
				varSelectionnees[i] := False;
				varRemoved[i] := False;
			end; { for i }

{ Debut de l'algorithme de recherche du R^2 le plus significatif }
		repeat
			etape := etape + 1;

			if etape > 1 then
				begin

					if isStepwise and (etape > 2) then
		{faire une/des procedure backward pour attester de la signification}
						begin
							for i := 1 to MAX_DESC do
								varBeforeAndAfter[i] := varSelectionnees[i];

		{ En cas de DTP ou TPT, quand on modifie la matrice Y, il faut re-remplir cette derniere }
		{ avec les donnees d'avant les permutations. }
		{ Nous ferons donc les permutations a l'aide du vecteur 'tempZ', meme s'il s'agit d'une }
		{ permutation matricielle. }
{ Mais ceci est fait dans la backward }
{       for i := 1 to nbValeurs do}
{       tempZ^[i] := tempY^[i];}

		{ Le test Backward }
							DoBackward(tempY, isStepwise);

		{ Comparons les rsultats du test  ceux que nous avions avant, et liminons }
		{ les variables qui n'ont pas t retenues: }
							for i := 1 to MAX_DESC do
								if (varBeforeAndAfter[i] <> varSelectionnees[i]) then
									varRemoved[i] := True;
						end;{if isStepwise}

{     if varSelectionnees[indiceRMaxSig] then}
{{enlever l'effet de la nouvelle variable selectionnee]}
{      begin}
{       for j := 1 to MAX_DESC do}
{       varSelTemp[j] := False;}
{       varSelTemp[indiceRMaxSig] := True;}
{       if not gQuitNow then}
{       DoPermTest(tempY, tempX, unB, unAddVect, varSelTemp, 0, etape, pToEnter, NO_PRINT, PARTIAL, '');}
{}
{       for i := 1 to nbValeurs do}
{       tempY^[i] := tempY^[i] - (unB^[1] * tempX^[i, indiceRMaxSig]);}
{}
{       for j := 1 to nbDescr do}
{{selectionner seulement la variable dont on desire enlever l'effet]}
{       varSelTemp[j] := False;}
{       varSelTemp[indiceRMaxSig] := True;}
{}
{       for i := 1 to nbDescr do}
{       if not varSelectionnees[i] then}
{       begin}
{       for j := 1 to nbValeurs do}
{       tempZ^[j] := tempX^[j, i];}
{       DoPermTest(tempZ, tempX, unB, unAddVect, varSelTemp, 0, etape, pToEnter, NO_PRINT, PARTIAL, '');}
{       for j := 1 to nbValeurs do}
{       tempX^[j, i] := tempX^[j, i] - (unB^[1] * tempX^[j, indiceRMaxSig]);}
{       end;{if not varSelectionnees[i]]}
{       NormalizeVect(tempY, False);{car les residus sont centres et non reduits]}
{       NormalizeMat(tempX, False);{car les residus sont centres et non reduits]}
{      end;{if varSelectionnees[indiceBMax]]}
				end;{if etape > 1}

{ Initialisation des compteurs et des "signets" (valeurs de comparaison) }
			probMinRSq := (pToEnter / etape);
			probMinB := MaxInt;
			indiceRMaxSig := 0;
			rSqMax := 0;

			NumToString(etape, str1);

			for i := 1 to nbDescr do
				if (not varSelectionnees[i]) and (not varRemoved[i]) and (not gQuitNow) then
					begin
						for j := 1 to nbDescr do
							varSelTemp[j] := False;

						varSelTemp[i] := True;

{ Note: on veut calculer 1) Quelle variable donne le R^2 le plus significatif }
{                        2) Quelle est la probabilite du b de cette variable }
{ Si le b est significatif (bonferroni), alors on inclut la variable. }


{{ On ne doit pas faire cela: on doit tester avec seulement la i-ieme variable car ]}
{{ on a deja enleve l'effet des variables selectionnes ]}
{ Non! on doit le faire car je viens de commenter le code qui elimine l'effet de la }
{ variable selectionnee }
						for j := 1 to nbDescr do
							if varSelectionnees[j] then
								varSelTemp[j] := not varRemoved[j];

{ Si on a deja choisi une variable dans cette etape, il ne faut pas l'inclure dans l'analyse! }
						if indiceRMaxSig > 0 then
							varSelTemp[indiceRMaxSig] := False;

						NumToString(i, str2);
						str2 := Concat('Step ', str1, ': testing significance of var. ', str2);

{ En cas de DTP ou TPT, quand on modifie la matrice Y, il faut re-remplir cette derniere }
{ avec les donnees d'avant les permutations. }
{ Nous ferons donc les permutations a l'aide du vecteur 'tempZ', meme s'il s'agit d'une }
{ permutation matricielle ("ordinaire"). }
						for j := 1 to nbValeurs do
							tempZ^[j] := tempY^[j];

						DoPermTest(tempZ, tempX, unB, unAddVect, varSelTemp, nbPerm, etape, pToEnter, NO_PRINT, PARTIAL, str2);
						TestQuit('Forward Selection');

{ Premier critere: }
{ Quelle variable donne le R^2 le plus significatif ? }
						probCurrRSq := (unAddVect.pgr + unAddVect.egr) / (nbPerm + 1);
						if probCurrRSq <= probMinRSq then
							begin
								probMinRSq := probCurrRSq;
{ Deuxieme critere: }
{ Le coefficient b de cette variable est-il significatif au seuil p-to-enter ? }
{ Notez le '* etape' -> Bonferonni! }

{ Trouver l'indice de la variable courante dans le vecteur qui nous }
{ a ete retourne du test par permutations }
								currVarIndex := 0;
								j := 0;
								repeat
									j := j + 1;
									if varSelTemp[j] = True then
										currVarIndex := currVarIndex + 1;
								until varSelTemp[j] > varSelectionnees[j];

								probCurrB := unAddVect.prob[currVarIndex];

								if probCurrB <= (pToEnter / etape) then {on accepte meme si c'est _exactment_ pToEnter}

									begin
										varSelectionnees[i] := True;

										if probCurrB < probMinB then
											probMinB := probCurrB
										else
{ Troisieme critere: }
{ Si 2 variables donnent un b aussi significatif, alors on prend le R^2 le plus grand }
											if probCurrB = probMinB then
{ Note: '>' pour eviter l'indecision; dans le cas d'egalite ou on prend la premiere variable... }
												if unAddVect.rSq > rSqMax then
													varSelectionnees[i] := True
												else
													varSelectionnees[i] := False
											else
												varSelectionnees[i] := False;

										if varSelectionnees[i] then
											begin
												probMinRSq := probCurrRSq;

												if indiceRMaxSig > 0 then { Si on a deja choisi une variable }
													varSelectionnees[indiceRMaxSig] := False;
												indiceRMaxSig := i;
												rSqMax := unAddVect.rSq;
{{ Pourquoi '1': car on ne teste qu'une variable a la fois et que ]}
{{ 'petitNbDescr' dans DoPermTest vaudra 1 ]}
{       addVectRMaxSig.eg[etape] := unAddVect.eg[1];}
{       addVectRMaxSig.pg[etape] := unAddVect.pg[1];}
{       addVectRMaxSig.egva[etape] := unAddVect.egva[1];}
{       addVectRMaxSig.pgva[etape] := unAddVect.pgva[1];}
{       addVectRMaxSig.prob[etape] := unAddVect.prob[1];}
{       addVectRMaxSig.probBonf[etape] := unAddVect.probBonf[1];}
{       addVectRMaxSig.rSq := unAddVect.rSq;}
{       addVectRMaxSig.pgr := unAddVect.pgr;}
{       addVectRMaxSig.egr := unAddVect.egr;}
{       bVarSel^[etape] := unB^[1];}
{ Eh non! C'est bien "etape", car on conserve les variables (sans enlever leur effet }
												for j := 1 to nbDescr do
													begin
														addVectRMaxSig.eg[j] := unAddVect.eg[j];
														addVectRMaxSig.pg[j] := unAddVect.pg[j];
														addVectRMaxSig.egva[j] := unAddVect.egva[j];
														addVectRMaxSig.pgva[j] := unAddVect.pgva[j];
														addVectRMaxSig.prob[j] := unAddVect.prob[j];
														addVectRMaxSig.probBonf[j] := unAddVect.probBonf[j];
														bVarSel^[j] := unB^[j];
													end; { for j }
												addVectRMaxSig.rSq := unAddVect.rSq;
												addVectRMaxSig.pgr := unAddVect.pgr;
												addVectRMaxSig.egr := unAddVect.egr;

											end; { if varSelectionnees[i] }

									end { if probCurrB <= pToEnter }

							end; { if probCurrRSq < probMinRSq }

					end;{if not varSelectionnees[i]}

{ L'indice de la _nouvelle_ variable donnant le R^2 le plus significatif et dont le b }
{ est significatif est maintenant compris dans 'indiceRMaxSig'. }


			if probMinB <= pToEnter then {on accepte meme si c'est _exactment_ pToEnter}
				begin
					if nbPerm > 0 then
						nbDecimales := Trunc(Ln(nbPerm) / Ln(10)) + 1
					else
						nbDecimales := 3;
					WriteLn(outputFile);

					WriteLn(outputFile, 'Step ', etape : 1, ':');

{ Imprimer la probabilite de Bonferroni }
					Write(outputFile, '     (*) Reference: Bonferonni-corrected alpha-level = ');
					WriteLn(outputFile, (pToEnter / etape) : 5 : 3);
					WriteLn(outputFile);

					WriteLn(outputFile, 'Variable  B Coefficient      EQ    GT    |EQ|    |GT|  Unil. prob.');
					WriteLn(outputFile, '-----------------------------------------------------  -----------');
					k := 0;
					for j := 1 to nbDescr do
						if varSelectionnees[j] then
							begin
								k := k + 1;
								Write(outputFile, ' X', j : 1, '         ');
								if bVarSel^[k] > 0 then
									Write(outputFile, ' ');
								Write(outputFile, bVarSel^[k] : 10 : 8, '    ');
								Write(outputFile, addVectRMaxSig.eg[k] : 4, '  ', addVectRMaxSig.pg[k] : 4, '   ');
								Write(outputFile, addVectRMaxSig.egva[k] : 4, '    ', addVectRMaxSig.pgva[k] : 4);
{ Les probabilites de Bonferronni sont inutiles dans ce cas-ci }
								Write(outputFile, '      ', addVectRMaxSig.prob[k] : (nbDecimales + 2) : nbDecimales);
								WriteLn(outputFile);
							end;
					WriteLn(outputFile);
					WriteLn(outputFile, '    R^2         EQ    GT    Prob ');
					WriteLn(outputFile, '------------   ----------  ------');
					Write(outputFile, ' ', addVectRMaxSig.rSq : 10 : 8, '   ');
					Write(outputFile, addVectRMaxSig.egr : 4, '  ', addVectRMaxSig.pgr : 4, '  ');
					WriteLn(outputFile, ((addVectRMaxSig.egr + addVectRMaxSig.pgr) / (nbPerm + 1)) : 6 : 4);
					WriteLn(outputFile);
					conditionArret := (etape > nbDescr);
				end{if probMin <= (pToEnter / etape}
			else
				conditionArret := True;

			if not conditionArret then
				begin
					WriteLn(outputFile, '-> Variable ', indiceRMaxSig : 1, ' significant at the ', pToEnter : 5 : 3, ' level.');
					WriteLn(outputFile);
				end;{if not}
		until conditionArret;

		if etape = 1 then
			begin
				WriteLn(outputFile, 'No variables were included at the ', pToEnter : 5 : 3, ' level.');
				WriteLn(outputFile);
			end; {if etape = 1}

		Dispose(tempX);
		Dispose(tempY);
		Dispose(tempZ);
		Dispose(unB);
		Dispose(bVarSel);

		WriteLn(outputFile, '--------------------------------');
		WriteLn(outputFile, 'End Forward Selection procedure.');
		WriteLn(outputFile, '--------------------------------');
		WriteLn(outputFile);
	end;{ procedure DoForward }

	procedure DoStepwise;
		var
			i, etape: Integer;
			lesDescr: ChoiceArray;
	begin
		pToEnter := SelectSignificanceLevel('Select the p-to-enter value', 2);
		pToRemove := SelectSignificanceLevel('Select the p-to-remove value', 1);

		WriteLn(outputFile, '------------------------------');
		WriteLn(outputFile, 'Stepwise Regression procedure:');
		WriteLn(outputFile, '------------------------------');
		WriteLn(outputFile);
		WriteLn(outputFile, 'p to enter: ', pToEnter : 5 : 3);
		WriteLn(outputFile, 'p to remove: ', pToRemove : 5 : 3);
		WriteLn(outputFile);

		DoForward(True);

		WriteLn(outputFile);
		WriteLn(outputFile, '----------------------------------');
		WriteLn(outputFile, 'End Stepwise Regression procedure.');
		WriteLn(outputFile, '----------------------------------');
		WriteLn(outputFile);
	end;{ procedure DoStepwise }

end.
; { Unit DoTests }
Dispose(unB);
Dispose(bVarSel);

WriteLn(outputFile, '--------------------------------');
WriteLn(outputFile, 'End Forward Selection procedure.');
WriteLn(outputFile, '--------------------------------');
WriteLn(outputFile);
end;{ procedure DoForward }

procedure DoStepwise;
	var
		i, etape: Integer;
		lesDescr: ChoiceArray;
begin
	pToEnter := SelectSignificanceLevel('Select the p-to-enter value', 2);
	pToRemove := SelectSignificanceLevel('Select the p-to-remove value', 1);

	WriteLn(outputFile, '------------------------------');
	WriteLn(outputFile, 'Stepwise Regression procedure:');
	WriteLn(outputFile, '------------------------------');
	WriteLn(outputFile);
	WriteLn(outputFile, 'p to enter: ', pToEnter : 5 : 3);
	WriteLn(outputFile, 'p to remove: ', pToRemove : 5 : 3);
	WriteLn(outputFile);

	DoForward(True);

	WriteLn(outputFile);
	WriteLn(outputFile, '----------------------------------');
	WriteLn(outputFile, 'End Stepwise Regression procedure.');
	WriteLn(outputFile, '----------------------------------');
	WriteLn(outputFile);
end;{ procedure DoStepwise }

end. { Unit DoTests }