unit MatrixAlgebra;

{**********}
interface
{**********}

	uses
		Sane;

	const
		MAX_OBJ = 29;	{ Maximum number of objects                 }
		MAX_DESC = 8;	{ Maximum number of descriptors             }
		MAX_VECT_LENGTH = 406;{ MAX_OBJ * (MAX_OBJ - 1) DIV 2       	    }

{ Types de permutations }
		kVectorPerm = 1;
		kMatrixPerm = 2;
		kDoublePerm = 3;
		kTriplePerm = 4;

	type
		Vector = array[1..MAX_VECT_LENGTH] of Double;
		VectorPtr = ^Vector;

		BigVectorInd = array[1..MAX_VECT_LENGTH] of Integer;
		BigVectorIndPtr = ^BigVectorInd;

		VectorInd = array[1..MAX_OBJ] of Integer;
		VectorIndPtr = ^VectorInd;

		Matrix = array[1..MAX_VECT_LENGTH, 1..MAX_DESC] of Double;
		MatrixPtr = ^Matrix;

		SqMatrix = array[1..MAX_DESC, 1..MAX_DESC] of Double;
		SqMatrixPtr = ^SqMatrix;

		SmallVector = array[1..MAX_DESC] of Double;
		SmallVectorPtr = ^SmallVector;

		ObjectVector = array[1..MAX_OBJ] of Double;
		ObjectVectorPtr = ^ObjectVector;

	var
		yInd, factSum: VectorIndPtr;
		yIndBig: BigVectorIndPtr;
		nbObjets, nbDescr, nbValeurs, nbPerm: Integer;
		gPermutationalMethod: Byte;

	procedure InitFactSum;

	function MtS (i, j: Integer): Integer;

	procedure StM (a: Integer;
									var i, j: Integer);

	function MDet (mat: SqMatrixPtr;
									n: Integer): Double;

	function MCofact (mat: SqMatrixPtr;
									n, line, col: Integer): Double;

	function MInverse (var mat: SqMatrixPtr;
									n: Integer): Boolean;

	procedure MtMMult (var aa: MatrixPtr;
									var cc: SqMatrixPtr;
									n, p: Integer);

	procedure MtVMult (var aa: MatrixPtr;
									var bb: VectorPtr;
									var cc: SmallVectorPtr;
									n, p: Integer);

	procedure MVMult (var aa: SqMatrixPtr;
									var bb, cc: SmallVectorPtr;
									n, p: Integer);

{****************}
implementation
{****************}

	procedure InitFactSum;
		var
			i: Integer;
	begin
		factSum^[1] := nbObjets - 1;
		for i := 2 to nbObjets do
			factSum^[i] := factSum^[i - 1] + nbObjets - i;
	end;

	function MtS (i, j: Integer): Integer;
		var
			temp: Integer;
	begin
		if i > j then {echanger les valeurs de i et j: triangulaire superieure}
			begin
				temp := i;
				i := j;
				j := temp;
			end;{if i > j}
{ A verifier... }
		if i = j then
			MtS := factSum^[i] - (nbObjets - j) + 1
		else
			MtS := factSum^[i] - (nbObjets - j);
	end;

	procedure StM (a: Integer;
									var i, j: Integer);
	begin
		i := 0;
		repeat
			i := i + 1
		until factSum^[i] >= a;
		j := nbObjets - (factSum^[i] - a);
	end;

	function MDet (mat: SqMatrixPtr;
									n: Integer): Double;
		var
			det, pivot: Double;
			i, j, k: Integer;
			tMat: SqMatrixPtr;

	begin
		if n > 1 then
			begin
				tMat := SqMatrixPtr(NewPtr(SizeOf(SqMatrix)));
{fill tMat with mat}
				for i := 1 to n do
					for j := 1 to n do
						tMat^[i, j] := mat^[i, j];
				for k := 1 to n - 1 do
					for i := k + 1 to n do {condensation pivotale pour la k-ime colonne}
						begin
							pivot := tMat^[i, k] / tMat^[k, k];
							for j := 1 to n do
								tMat^[i, j] := tMat^[i, j] - (pivot * tMat^[k, j]);
						end;
				det := tMat^[1, 1];
				for i := 2 to n do
					det := det * tMat^[i, i];
				Dispose(tMat);
				MDet := det;
			end {if n > 1}
		else
			MDet := mat^[n, n];
	end; {MDet}

	function MCofact (mat: SqMatrixPtr;
									n, line, col: Integer): Double;
		var
			cof: Double;
			i, j, ii, jj, signe: Integer;
			tMat: SqMatrixPtr;

	begin
		if (n = 2) then
			begin
				signe := ((-1) * ((line + col) mod 2));
				if (signe = 0) then
					signe := 1;
				cof := signe * mat^[(n + 1) - line, (n + 1) - col];
			end
		else {calculer le cofacteur pour vrai!}
			begin
				tMat := SqMatrixPtr(NewPtr(SizeOf(SqMatrix)));
{fill tMat with mat, excluding one row and one column}
				ii := 0;
				for i := 1 to n do
					if i <> line then
						begin
							ii := ii + 1;
							jj := 0;
							for j := 1 to n do
								begin
									if j <> col then
										begin
											jj := jj + 1;
											tMat^[ii, jj] := mat^[i, j];
										end;{if j}
								end;{for j}
						end;{if i}
				signe := ((-1) * ((line + col) mod 2));
				if (signe = 0) then
					signe := 1;
				cof := signe * MDet(tMat, n - 1);
				Dispose(tMat);
			end;
		MCofact := cof;
	end;{MCofact}

	function MInverse (var mat: SqMatrixPtr;
									n: Integer): Boolean;
		var
			tMat: SqMatrixPtr;
			i, j: Integer;
			det: Double;
	begin
		if n > 1 then
			begin
				tMat := SqMatrixPtr(NewPtr(SizeOf(SqMatrix)));
{save mat in a temporary matrix}
				for i := 1 to n do
					for j := 1 to n do
						tMat^[i, j] := mat^[i, j];
				det := MDet(tMat, n);
				if det = 0 then
					MInverse := False {matrice singulire, non-inversible}
				else
					begin {debut inversion}
						MInverse := True;
						det := 1 / det;
						for i := 1 to n do
							for j := 1 to n do
								mat^[i, j] := det * MCofact(tMat, n, j, i);
					end;{inversion}
				Dispose(tMat);
			end{if n > 1}
		else if n = 1 then
			if mat^[1, 1] <> 0 then
				begin
					mat^[1, 1] := 1 / mat^[1, 1];
					MInverse := True;
				end {if mat^[1, 1] <> 0}
			else
				MInverse := False {car le seul element de la matrice est nul}
		else
			MInverse := False;{car n < 1!!!}
	end;{MInverse}

	procedure MtMMult (var aa: MatrixPtr;
									var cc: SqMatrixPtr;
									n, p: Integer);
		var
			i, j, k: Integer;
			tempSum: Double;
	begin
		for i := 1 to p do
			begin
				for j := 1 to p do
					begin
						tempSum := 0;
						for k := 1 to n do
							begin
								tempSum := tempSum + aa^[k, i] * aa^[k, j];
							end;
						cc^[i, j] := tempSum;
					end;
			end;
	end; { Procedure MtMMult}

	procedure MtVMult (var aa: MatrixPtr;
									var bb: VectorPtr;
									var cc: SmallVectorPtr;
									n, p: Integer);
		var
			i, j, k, tempI, tempJ, tempIndice: Integer;
			tempSum: Double;
	begin
		for i := 1 to p do
			begin
				tempSum := 0;
				for k := 1 to n do
					if gPermutationalMethod <> kVectorPerm then
						begin
							tempIndice := k;
							StM(tempIndice, tempI, tempJ);
							tempIndice := MtS(yInd^[tempI], yInd^[tempJ]);
							tempSum := tempSum + aa^[k, i] * bb^[tempIndice];{car bb est y}
						end { if }
					else { Permutations "normales", i.e. pas Mantel }
						begin
{       tempI := MtS(i, k);}
{       tempIndice := yIndBig^[tempI];}
							tempIndice := yIndBig^[k];
							tempSum := tempSum + aa^[k, i] * bb^[tempIndice];{car bb est y}
						end; { else }
				cc^[i] := tempSum;
			end;
	end; { Procedure MtVMult}

	procedure MVMult (var aa: SqMatrixPtr;
									var bb, cc: SmallVectorPtr;
									n, p: Integer);
		var
			i, j, k: Integer;
			somme: Double;
	begin
		for i := 1 to n do
			begin
				somme := 0;
				for j := 1 to p do
					somme := somme + aa^[i, j] * bb^[j];
				cc^[i] := somme;
			end;
	end; { Procedure MVMult }

end. {implementation of MatrixAlgebra}