program PermuteMain;

{ Version 3.2 - Multiple Regression tested with permutations, with ter Braak correction }
{ Part of the code is from 'SuperMantel' and 'Procruste'.    }

{ 3.1.1: Program now works correctly with System 6 }
{ 3.2: ter Braak correction (F-tests) }

	uses
		MachineSpecific, MatrixAlgebra, Permute, DoTests;

	procedure Main;
		var
			i, j, choixTest, nombre, numTest: Integer;
			yNom, xNom, outputFileName, tempStr: Str255;
			binChoix: BinStrArray;
			lesDescr: ChoiceArray;
			choixMethStr: Str32;
			min: Double;{pour recherche dans le vecteur d'enracinement}
	begin
		Y := VectorPtr(NewPtr(SizeOf(Vector)));
		X := MatrixPtr(NewPtr(SizeOf(Matrix)));
		B := SmallVectorPtr(NewPtr(SizeOf(SmallVector)));

{  New(addVect);}
		indVect := VectorIndPtr(NewPtr(SizeOf(VectorInd)));

		gPrintedNotes := False;
		gRoot := 0;{ No object is the root of the tree }

		nbObjets := 0;
		nbValeurs := 0;
{  if gPermutationalMethod = kSimplePerm then}
{   nbValeurs := GetInteger('Number of values', '(number of lines in the input files)', 1, MAX_VECT_LENGTH)}
{  else}
		nbObjets := GetInteger('Number of objects', '(it is NOT the number of lines in the input files)', 1, MAX_OBJ);
		if (nbObjets = NOTHING) or (nbValeurs = NOTHING) then
			Quit;

		nbDescr := GetInteger('Number of descriptors', '(independent variables)', 1, MAX_DESC);
		if nbDescr = NOTHING then
			Quit;

		factSum := VectorIndPtr(NewPtr(SizeOf(VectorInd)));
		InitFactSum;

		nombre := 0;
		while not (nombre = 1) do
			begin
				nombre := GetName('File containing the unfolded Y matrix', TEXT_TYPE, yNom);
				if nombre <> 0 then
					Quit;
				MyReset(yFile, yNom);
				if nbValeurs = 0 then {au cas ou on ne fait pas de perm. simple}
					nbValeurs := (nbObjets * (nbObjets - 1) div 2);
				SetupProgress('Reading vector Y', 0, nbValeurs);
				i := 1;
				while (i <= nbValeurs) and (not gQuitNow) do
					begin
						ProgressInfo(i);
						ReadLn(yFile, Y^[i]);
						i := i + 1;
					end; {while i}
				Close(yFile);
				DisposeProgress(gWorkingFromFile);
				if not gQuitNow then
					nombre := 1;
			end;{while not (nombre=1)}

		nombre := 0;
		while not (nombre = 1) do
			begin
				nombre := GetName('File containing the X matrices in columns', TEXT_TYPE, xNom);
				if nombre <> 0 then
					Quit;
				MyReset(xFile, xNom);
				SetupProgress('Reading X matrices', 0, nbValeurs);
				i := 1;
				while (i <= nbValeurs) and (not gQuitNow) do
					begin
						ProgressInfo(i);
						for j := 1 to nbDescr do
							Read(xFile, X^[i, j]);
						ReadLn(xFile);
						i := i + 1;
					end;{while i}
				Close(xFile);
				DisposeProgress(gWorkingFromFile);
				if not gQuitNow then
					nombre := 1;
			end;{while not (nombre = 1)}

		nombre := GetOutputName('Output File', outputFileName);
		if nombre <> 0 then
			Quit;
		MyReWrite(outputFile, outputFileName);
		outputFileOpen := True;

		for numTest := 1 to numberOfConsecutiveTests do
			begin
				tempStr := Concat('Suggestion: 99, 499, 999', cr, '0 = no permutations');{Necessaire pour la fonction suivante}
				nbPerm := GetInteger('Number of permutations', tempStr, 0, MaxInt);
				if nbPerm = NOTHING then
					Quit;

				if nbPerm > 0 then
					begin
						lesDescr[kVectorPerm] := 'Vector-type';
						lesDescr[kMatrixPerm] := 'Matrix-type';
						lesDescr[kDoublePerm] := 'Double (DPT)';
						lesDescr[kTriplePerm] := 'Triple (TPT)';
						choixTest := 1;
						GetExclusiveChoice('Select permutational method:', kTriplePerm, lesDescr, choixTest);
						gPermutationalMethod := Byte(choixTest);
						choixMethStr := lesDescr[gPermutationalMethod];

						case gPermutationalMethod of
							kVectorPerm, kMatrixPerm: 
								;
							kDoublePerm: 
								begin
									ultrametricY := VectorPtr(NewPtr(SizeOf(Vector)));
									for i := 1 to nbValeurs do
										ultrametricY^[i] := Y^[i];
								end;{kDoublePerm}
							kTriplePerm: 
								begin
									additiveY := VectorPtr(NewPtr(SizeOf(Vector)));
									ultrametricY := VectorPtr(NewPtr(SizeOf(Vector)));
									starY := VectorPtr(NewPtr(SizeOf(Vector)));
									rootVectY := ObjectVectorPtr(NewPtr(SizeOf(ObjectVector)));
									rootVectInd := VectorIndPtr(NewPtr(SizeOf(VectorInd)));

{ Keep the raw data for future reference }
									for i := 1 to nbValeurs do
										additiveY^[i] := Y^[i];

{select the rooting object}
									i := GetInteger('Rooting Object', '(object that roots the additive tree)', 1, nbObjets);
									gRoot := Byte(i);
{extract the rooting vector}
									nombre := gRoot - 1;
									if gRoot = 1 then
										rootVectY^[1] := 0
									else
										rootVectY^[1] := Y^[nombre - 1];
									for i := 2 to nbObjets do
										begin
											if i > gRoot then {move right}
												begin
													nombre := nombre + 1;
													rootVectY^[i] := Y^[nombre];
												end;{>}
											if i < gRoot then {move down}
												begin
													nombre := nombre + nbObjets - i;
													rootVectY^[i] := Y^[nombre];
												end;{<}
											if i = gRoot then {move diagonally}
												begin
													nombre := nombre + nbObjets - i;
													rootVectY^[i] := 0;
												end;{=}
										end;{for i}
{ Find the minimum in the rooting vector }
									min := MaxLongInt; { A sufficiently large number! }
									for i := 1 to nbObjets do
										if i <> gRoot then
											if min > rootVectY^[i] then
												min := rootVectY^[i];
{ And substract it from the vector }
									for i := 1 to nbObjets do
										if i <> gRoot then
											rootVectY^[i] := rootVectY^[i] - min;
{ Now we have a rooting vector that will allow us to create the star tree matrix }
{ We should initialize its indirection vector }
									for i := 1 to nbObjets do
										rootVectInd^[i] := i;
								end;{kTriplePerm}
						end;{case}
					end;{if nbPerm > 0}

				if numTest = 1 then
					begin
{ We normalize the data anyways }
{      binChoix[1] := 'Yes';}
{      binChoix[2] := 'No';}
{      tempStr := Concat('Is your data standardized ?', cr, '(if not, it will be)');}
{      gDataStandardized := BinaryChoice(tempStr, binChoix);}
{      if not gDataStandardized then}
{       begin}
						NormalizeMat(X, True);{Normalize the X matrix}
						NormalizeVect(Y, True);
{     end;{if not gDataStandardized]}

						WriteLn(outputFile, PROGRAM_NAME);
						WriteLn(outputFile, 'By Philippe Casgrain  1993');
						WriteLn(outputFile);
						WriteLn(outputFile, 'Y Vector:   "', yNom, '"');
						WriteLn(outputFile, 'X Matrices: "', xNom, '"');
						WriteLn(outputFile, 'Number of permutations: ', nbPerm : 1);
						WriteLn(outputFile, 'Permutational method: ', choixMethStr);
						if gPermutationalMethod = kTriplePerm then
							WriteLn(outputFile, 'Rooting object: ', gRoot : 1);
					end;{if numTest = 1}
				if numberOfConsecutiveTests > 1 then
					begin
						WriteLn(outputFile);
						WriteLn(outputFile, '*** A N A L Y S I S   ', numTest : 1, ' ***');
					end;{if numberOfConsecutiveTests > 1}
				WriteLn(outputFile);

				lesDescr[kTestSimple] := 'Simple Test';
				lesDescr[kBackward] := 'Backward Elimination';
{    lesDescr[kForward] := 'Forward Selection';}
				lesDescr[kForward] := 'Forward Selection';
				lesDescr[kStepwise] := 'Stepwise Regression';
				choixTest := 1;
				if nbPerm > 0 then
					GetExclusiveChoice('Please select one:', kStepwise, lesDescr, choixTest);

				case choixTest of
					kTestSimple: 
						DoTestSimple;
					kBackward: 
						DoBackward(Y, False);
{     kForward: }
{      DoOldForward(False);}
					kStepwise: 
						DoStepwise;
					kForward: 
						DoForward(False);
					otherwise
						Quit;
				end;{case}

			end;{for numTest := 1 to numberOfConsecutiveTests}

		Dispose(Y);
		Dispose(X);
		Dispose(B);
		Dispose(factSum);
{  Dispose(addVect);}
		case gPermutationalMethod of
			kVectorPerm, kMatrixPerm: 
				;
			kDoublePerm: 
				Dispose(ultrametricY);
			kTriplePerm: 
				begin
					Dispose(additiveY);
					Dispose(ultrametricY);
					Dispose(starY);
					Dispose(rootVectY);
					Dispose(rootVectInd);
				end;{kTriplePerm}
			otherwise
				;
		end;
		Dispose(indVect);
		Quit;
	end;{procedure Main}


begin

 { Only one test; this may change if we're working from a batch file }
	numberOfConsecutiveTests := 1;
	Initialize;

	Presentation;
	repeat
		if not gQuitNow then
			Main;
	until not gDaemonMode;

end.